' SCP1000 barometer test program
' Writes temperature and barometric pressure (in PA)
'
' Adaptation from Don Kinzer's base code by Jurjen Kranenborg:
' 	- OpenSPI flags register setting corrected (Clock-phase false, max 500 KHz bus)
'	- Implementation of SCP1000 initialization
'   - correction of Read_Direct_Access function for cnt=1 case (high - low byte positioning)
' (Adaptions commented in code below)
' 
' Program tested OK on a ZX328L setup with a SparkFun (c) SCP1000 SPI breakout board connected


' SPI interface defs:
Const csPin as Byte = B.2 
Const flags as Byte = &b00000010  ' OpenSPI setup with clock phase False, SCK must be <= 500KHz (use f/64) 
Const chanSPI as Byte = 1 
Const DrdyPin as Byte = B.1 
' Memory space for handling register data
Const TempReg as Byte = &H21 
Const PresRegHi as Byte = &H1f 
Const PresRegLo as Byte = &H20 


' Read from byte or 16-bit word data registers
Function Read_Direct_Access(ByVal addr as Byte, ByVal cnt as Byte) as UnsignedInteger 
  Dim dataIn(1 to 2) as Byte 
  dataIn(1) = 0 
  dataIn(2) = 0 
  addr = Shl(addr, 2) 
  If (cnt > 2) Then 
    cnt = 2 
  End If 
  Call SPICmd(chanSPI, 1, addr, cnt, dataIn) 
  ' Be aware to handle cnt=1 differently from cnt = 2
  ' If cnt = 2 then the high byte (transmitted first) ends up in DataIn(1)
  ' If cnt = 1 then the high byte should be zero and the low byte is the one read
  If (cnt = 2) Then
	Read_Direct_Access = MakeWord(dataIn(2), dataIn(1)) 
  Else
	Read_Direct_Access = MakeWord(dataIn(1), &H00)
  End If
End Function 

' Write to byte register
Sub Write_Direct_Access(ByVal addr as Byte, ByVal data as Byte) 
  Dim dataOut(1 to 2) as Byte 
  
  dataOut(1) = Shl(addr, 2) Or &H02 
  dataOut(2) = data 
  Call SPICmd(chanSPI, 2, dataOut, 0, 0) 
End Sub

Sub init_Scp1000()
  Call OpenSPI(chanSPI, flags, csPin) 	' Describe SCP1000 SPI protocol characteristics
  Delay (0.1)
  Call Write_Direct_Access(&H06, &H01)	' Software reset
  Delay (0.1)
  Call Write_Direct_Access(&H03, &H0A)	' Operation mode: continous + high resolution
  Delay (0.1)
End Sub



Sub Main() 

  ' SCP1000 initialization 
  Call init_Scp1000() 
  
  ' loop reading temperature and pressure 
  Do 
    Do While Not CBool(GetPin(DrdyPin)) 
    Loop 
	
    Dim temperature as Integer 
    Dim pressureLo as UnsignedInteger 
    Dim pressureHi as UnsignedInteger 
    Dim pressure as UnsignedLong 

    ' read the temperature register, sign-extend the result 
    '   the units will be 1/20 degree Celsius 
    temperature = CInt(Read_Direct_Access(TempReg, 2)) 
    If CBool(HiByte(temperature) And &H20) Then 
      temperature = temperature Or &Hc000 
    End If 
    
    ' read the high byte and low word of the pressure value 
    pressureHi = Read_Direct_Access(PresRegHi, 1)
    pressureLo = Read_Direct_Access(PresRegLo, 2) 	

    ' combine the values, units will be quarter Pascals 
    pressure = MakeDword(pressureLo, pressureHi) 

    ' display the temperature and pressure 
    Debug.Print "Temperature: "; Fmt(CSng(temperature) / 20.0, 2); " *C" 
    Debug.Print "Pressure:    "; Fmt(CSng(pressure) / 4.0, 2); " Pa" 
    
  Loop 
End Sub